#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <ctime>

#include "sbutils.h"
#include "sbsmtpclient.h"
#include "sbmime.h"
#include "sbmisc.h"
#include "sbcertvalidator.h"

using namespace SecureBlackbox;

std::string getDomain(std::string email)
{
    int p = email.find('@');
    return email.substr(p);
}

std::string GetFilename(std::string const& path)
{
	std::string result;
	size_t pos = path.find_last_of("\\");
	if(pos != std::string::npos)
		result.assign(path.begin() + pos + 1, path.end());
	else
		result = path;

	return result;
}

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

std::wstring StrToWStr(std::string str)
{
	std::wstring res ( str.begin(), str.end() );
	return res;
}

void SB_CALLBACK CertificateValidateEvent(void * _ObjectData, TObjectHandle Sender, TElX509CertificateHandle X509Certificate, int8_t * Validate)
{
	TElX509Certificate Certificate(X509Certificate, false);
	TSBCertificateValidity Validity = cvInvalid;
	TSBCertificateValidityReason Reason;
	if ((Certificate.get_Chain() == NULL) || (Certificate.get_Chain()->get_Certificates(0)->getHandle() == X509Certificate))
	{
		// For proper CRL and OCSP validation please read instructions in 
		// description of ElX509CertificateValidator class in the help file
		std::time_t t = std::time(0);
		TElX509CertificateValidator CertificateValidator(NULL);
		CertificateValidator.ValidateForSSL(&Certificate, *static_cast<std::string*>(_ObjectData), "", hrServer, NULL, false, t, Validity, Reason);
		*Validate = (Validity == cvOk) || (Validity == cvSelfSigned);
		if (!(*Validate)) {
			std::cout << "There was a problem validating the server certificate. Do you wan't to continue? : Y or N" << std::endl;

			std::string input;
			std::getline(std::cin, input);
			std::transform(input.begin(), input.end(), input.begin(), ::toupper);

			*Validate = ((input == "Y") ? true : false);
		}
	}
	else
		*Validate = true;
}

int main(int argc, char **argv)
{
    std::string helpMessage = "consolesmtp.exe [params]\n" \
        "[params]\n" \
        "-from : sender e-mail;\n" \
        "-to : addressee e-mail;\n" \
        "-subject : message title;\n" \
        "-body : message content;\n" \
        "-attachment : file to attach;\n" \
        "-server : smtp server;\n" \
        "-port : port;\n" \
        "-ssl : \"nossl\", \"explicit\" or \"implicit\" mode;\n" \
        "-username : username;\n" \
        "-password : password;\n" \
        "-h : help;\n";
	std::string to, from, subject, bodystr, attachment, server, port, ssl, username, password;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << E.what() << std::endl;
		return 0;
	}

	try
	{
		int i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-from")
			{
				if (++i < argc)
					from = UnQuote(argv[i]);
			}
			else if (param == "-to")
			{
				if (++i < argc)
					to = UnQuote(argv[i]);
			}
			else if (param == "-subject")
			{
				if (++i < argc)
					while ((i <= argc) && (argv[i + 1][0] != '-'))
					{
						i++;
						subject += argv[i];
					}
			}
			else if (param == "-body")
			{
				if (++i < argc)
					while ((i <= argc) && (argv[i + 1][0] != '-'))
					{
						i++;
						bodystr += argv[i];
					}
			}
			else if (param == "-attachment")
			{
				if (++i < argc)
					attachment = UnQuote(argv[i]);			
			}
			else if (param == "-server")
			{
				if (++i < argc)
					server = UnQuote(argv[i]);			
			}
			else if (param == "-port")
			{
				if (++i < argc)
					port = UnQuote(argv[i]);			
			}
			else if (param == "-ssl")
			{
				if (++i < argc)
					ssl = UnQuote(argv[i]);			
			}
			else if (param == "-username")
			{
				if (++i < argc)
					username = UnQuote(argv[i]);			
			}
			else if (param == "-password")
			{
				if (++i < argc)
					password = UnQuote(argv[i]);			
			}
			else if (param == "-h")
			{
				std::cout << helpMessage << std::endl;
			}

			i++;
		}
	}
	catch (SBException E)
	{
		std::cout << "Parameters parsing error:" << E.what() << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	if (from == "")
	{
		std::cout << "Parameter 'from' is empty" << std::endl;
		return 0;
	}

	if (to == "")
	{
		std::cout << "Parameter 'to' is empty" << std::endl;
		return 0;
	}

	if (bodystr == "")
	{
		std::cout << "Parameter 'body' is empty" << std::endl;
		return 0;
	}

	if (server == "")
	{
		std::cout << "Parameter 'server' is empty" << std::endl;
		return 0;
	}

	if (port == "")
	{
		std::cout << "Parameter 'port' is empty" << std::endl;
		return 0;
	}

	if (username == "")
	{
		std::cout << "Parameter 'username' is empty" << std::endl;
		return 0;
	}

	if (password == "")
	{
		std::cout << "Parameter 'password' is empty" << std::endl;
		return 0;
	}

	try
	{
		TElSMTPClient smtp (NULL);
		smtp.set_Versions((TSBVersions)(f_sbTLS1 | f_sbTLS11 | f_sbTLS12));
		std::wstring mailer = L"";
		TElMessage msg (false, mailer);
		TElMessagePart mmp (&msg, NULL);
		TElPlainTextPart body;
		
		msg.SetMainPart(mmp, false);
		body.SetText(StrToWStr(bodystr));
		mmp.AddPart(body);
     
		TElMailAddressList* alTo = msg.get_To_();
		alTo->AddAddress(L"", StrToWStr(to));
		TElMailAddressList* alFrom = msg.get_From();
		alFrom->AddAddress(L"", StrToWStr(from));
		msg.SetSubject(StrToWStr(subject));
		std::wstring messageId;
		TElMessage::GenerateMessageID(messageId);
		msg.set_MessageID(messageId);
		std::time_t t = std::time(0);
		msg.SetDate(t);
		
		if (attachment != "")
		{
			std::string fileName = GetFilename(attachment);
			TElMessagePart attach (msg, mmp);
			TFileStream fs (attachment, filemodeOpenRead);

			mmp.AddPart(attach);

			TElMessageHeader* header = attach.get_Header();

			TElMessageHeaderField mhf1 (header->AddField(L"Content-Type", L"application/octet-stream", true), NULL);
			mhf1.AddParam(L"name", StrToWStr(fileName));

			TElMessageHeaderField mhf2 (header->AddField(L"Content-Disposition", L"attachment", true), NULL);
			mhf2.AddParam(L"filename", StrToWStr(fileName));

			header->AddField(L"Content-Transfer-Encoding", L"base64", true);
			attach.SetData(fs, (int)fs.get_Size(), false);
		}
		smtp.set_OnCertificateValidate(&CertificateValidateEvent, &server);
		smtp.set_Address(server);
		smtp.set_Port(atoi(port.c_str()));
		if (ssl == "nossl")
		{
			smtp.set_UseSSL(false);
		}
		else if (ssl == "implicit")
		{
			smtp.set_UseSSL(true);
			smtp.set_SSLMode(smImplicit);
		}
		else
		{
			smtp.set_UseSSL(true);
			smtp.set_SSLMode(smExplicit);
		}
		smtp.set_Username(username);
		smtp.set_Password(password);

		smtp.Open();
		smtp.Login(getDomain(from));
		smtp.Send(msg);
		smtp.Close(false);
	}
	catch (SBException E)
	{
		std::cout << "Error:" << E.what() << std::endl;
	}

	return 0;
}